# 2022.8.10
# 最適化モデルのためのデータ作成
# CutとFillで土砂量の合計値を同じにする

# import
import os
import pandas as pd
import numpy as np
import glob


# set dir
os.chdir("E:/cw/202202_hara/202202_土砂移動/最適化/大阪府土砂条例のみ")
# os.getcwd()

###################
# 土砂条例用
###################
# read file
cut1 = pd.read_csv("cut_no1.csv" ,encoding="cp932")
cut2 = pd.read_csv("cut_no2.csv" ,encoding="cp932")
fill = pd.read_csv("fill.csv" ,encoding="cp932")

# cut1とcut2を結合
cut = pd.concat((cut1,cut2))
# cut.columns
# fill.columns

# 番号を取得（読み込み時に番号が日付に変換される場合がある）
c_num = sorted(set(cut["番号"].values))
f_num = sorted(set(fill["番号"].values))
c_num == f_num # 中身が同じか確認する

########
# fill
########
# fillデータの成型
fill_dat = fill[['番号', '埋立て等区','LocName', 'fX', 'fY']]
fill_dat = fill_dat.drop_duplicates(keep = "first", ignore_index =True)  # 重複行を削除してインデックスを振り直す

# 番号毎に搬入量（cut）と搬出量（fill）を合わせる
cut_sum = []
for i in range(len(fill_dat)):
    id = fill_dat["番号"][i]
    cut_sum.append(np.sum(cut[cut["番号"] == id]["今回報告量(t)"])) # 番号毎の搬出量を合計

fill_dat["土砂搬入量(t)"] = cut_sum


# 搬入場所で合計する（同じ搬入場所でも番号が違うものがあるため）
# fillデータの成型
fill_dat2 = fill_dat[['埋立て等区','LocName', 'fX', 'fY']]
fill_dat2 = fill_dat2.drop_duplicates(keep = "first", ignore_index =True)  # 重複行を削除してインデックスを振り直す

# 搬入場所毎に搬入量をまとめる
fill_sum = []
for i in range(len(fill_dat2)):
    f_place = fill_dat2["埋立て等区"][i]
    fill_sum.append(np.sum(fill_dat[fill_dat["埋立て等区"] == f_place]["土砂搬入量(t)"])) # 番号毎の搬出量を合計

fill_dat2["土砂量t"] = fill_sum
fill_dat2 = fill_dat2.rename(columns={'埋立て等区': 'Loc'})

# csvに出力
fill_dat2.to_csv("fill.csv",encoding="cp932")

# 確認
# tmp_cut = np.sum(cut["今回報告量(t)"])
# tmp_fill = np.sum(fill_dat2["土砂量t"])


########
# cut
########
# idを作成
ids=[]
for id in cut1["id"]:
    ids.append("1_" + str(id))
cut1["c_id"] = ids

ids=[]
for id in cut2["id"]:
    ids.append("2_" + str(id))
cut2["c_id"] = ids

# cut1とcut2を結合
cut = pd.concat((cut1,cut2))
cut.to_csv("cut_12.csv",encoding="cp932")

# cutデータの成型
cut_dat = cut[['c_id','土砂の発生', 'LocName', 'fX', 'fY',"今回報告量(t)"]]
cut_dat = cut_dat[cut_dat["今回報告量(t)"] > 0] # 土砂量が0のものを省く
cut_dat = cut_dat.rename(columns={'今回報告量(t)': '土砂量t','土砂の発生': 'Loc','fX': 'cX','fY': 'cY'})

# csvに出力
cut_dat.to_csv("cut_osaka.csv",encoding="cp932")

# 確認
# tmp_cut2 = np.sum(cut_dat["土砂量(t)"])



###################
# ちぎりアイランド用
###################
# set dir
os.chdir("E:/cw/202202_hara/202202_土砂移動/最適化/ちきりアイランド")

# ファイル名取得
files = glob.glob('cut_2*.csv')

# 列名を設定
col_name = ['ID','場所', 'sX', 'sY',"搬入量"]

# 全てのファイルを読み込む
cut_data = []
for f in files:
    # read file
    dat = pd.read_csv(f , encoding="cp932")
    dat = dat[col_name] # 必要な部分のみを抜き出す

    cut_data.append(dat)

# データをまとめる
cut_data = pd.concat(cut_data)
cut_data = cut_data.rename(columns={'ID': 'c_id','搬入量': '土砂量(t)','sX': 'cX','sY': 'cY'}) # 列名を変更

# csvに出力
cut_data.to_csv("cut_chikiri.csv",encoding="cp932")

################
# Join cut
################
# set dir
os.chdir("E:/cw/202202_hara/202202_土砂移動/最適化")

# read file
# cut data
cut_chikiri = pd.read_csv("cut_chikiri.csv" ,encoding="cp932")
cut_osaka = pd.read_csv("cut_osaka.csv" ,encoding="cp932")

# 大阪と岸和田データをまとめる
cut = pd.concat((cut_osaka,cut_chikiri))
cut = cut.iloc[:,1:]
cut = cut.reset_index(drop=True)
cut = cut.reset_index(drop=False)
cut = cut.rename(columns={'index': 'c_index'})

cut.to_csv("cut.csv",encoding="cp932")



################
# Line用ファイル作成
################
# 土砂条例データとちきりデータをマージして、埋立地と土砂発生場所の総当たり表を作成

# set dir
os.chdir("E:/cw/202202_hara/202202_土砂移動/最適化")

# read file
# cut data
cut = pd.read_csv("cut.csv" ,encoding="cp932")

# fill data
fill = pd.read_csv("fill.csv" ,encoding="cp932")
fill = fill.iloc[:,1:]
fill = fill.reset_index(drop=False)
fill = fill.rename(columns={'index': 'f_index'})


# cutデータを1行づつ取り出してfill行分複製する
cuts = []
for i in range(len(cut)):
    sel_cut = cut.iloc[cut.index == i]
    sel_cut = sel_cut.append([sel_cut]*(len(fill)-1), ignore_index=True) # 行の複製
    cuts.append(sel_cut)

cuts = pd.concat(cuts)
cuts = cuts.reset_index(drop=True)

# fillデータをcut行分複製する
fills = fill.append([fill]*(len(cut)-1),ignore_index=True) # 行の複製

# 複製したデータを結合する
line = pd.concat((cuts, fills),axis=1)
# csvに出力
line.to_csv("line_km.csv",encoding="cp932")





# ################
# # cut1:同じ場所はまとめる
# ################
#
# cut_dat = cut[['土砂の発生', 'LocName', 'fX', 'fY']]
# cut_dat = cut_dat.drop_duplicates(keep = "first", ignore_index =True)  # 重複行を削除してインデックスを振り直す
#
# # 番号毎に搬入量と搬出量を合わせる
# cut_sum2=[]
# for i in range(len(cut_dat)):
#     c_place = cut_dat["土砂の発生"][i]
#     cut_sum2.append(np.sum(cut[cut["土砂の発生"] == c_place]["今回報告量(t)"])) # 番号毎の搬出量を合計
#
# cut_dat["土砂量(t)"] = cut_sum2
# cut_dat = cut_dat[cut_dat["土砂量(t)"]>0]
# cut_dat = cut_dat.drop_duplicates(keep = "first", ignore_index =True)  # 重複行を削除してインデックスを振り直す
#
# # csvに出力
# cut_dat.to_csv("cut_sum.csv",encoding="cp932")
# # tmp_cut2 = np.sum(cut_dat["土砂量(t)"])

#
# ################
# # Line用ファイル作成
# ################
# # read file
# cut = pd.read_csv("cut_sum_gis.csv" ,encoding="cp932")
# fill = pd.read_csv("fill_sum_gis.csv" ,encoding="cp932")
#
# # col_nameを変更
# cut = cut.rename(columns={'fX': 'cX', 'fY': 'cY'})
#
#
# cuts = []
# for i in range(len(cut)):
#     sel_cut = cut.iloc[cut.index==i]
#     sel_cut =  sel_cut.append([sel_cut]*(len(fill)-1),ignore_index=True)
#     cuts.append(sel_cut)
#
# cuts = pd.concat(cuts)
# cuts = cuts.reset_index(drop=True)
#
# fills = fill.append([fill]*(len(cut)-1),ignore_index=True)
#
# line = pd.concat((fills,cuts),axis=1)
# line["id"] = line["f_id"] + "_" +line["c_id"]
#
#
# line.to_csv("line.csv",encoding="cp932")
